#ifndef _PR2_JOINT_SPACE_DDP_EX_
#define _PR2_JOINT_SPACE_DDP_EX_


#include <cmath>
#include <algorithm>
#include <string>

#include <ros/ros.h>

#include <kinematics_msgs/GetKinematicSolverInfo.h>
#include <kinematics_msgs/GetPositionFK.h>
#include <kinematics_msgs/GetPositionIK.h>

#include <mpc/types.h>

using namespace MPC;

namespace pr2_joint_space {
        
        class RightArmModel {
        private:
// the goal in euclidean coordinates
                VectorT goal_;
                VectorT res_;
                VectorT external_goal_;
                ros::NodeHandle &n_;
                ros::ServiceClient fk_info_client_;
                ros::ServiceClient fk_client_;
                ros::ServiceClient ik_client_;
                std::vector<std::string> controlled_joints_;
                std::vector<int> joint_mapping_;

                // cached temporaries
                kinematics_msgs::GetKinematicSolverInfo::Request request;
                kinematics_msgs::GetKinematicSolverInfo::Response response;
                kinematics_msgs::GetPositionFK::Request  fk_request;
                kinematics_msgs::GetPositionFK::Response fk_response;
                kinematics_msgs::GetPositionIK::Request  ik_request;
                kinematics_msgs::GetPositionIK::Response ik_response;
                
                // some constants
                float h;
                float c_u;
                float c_xT;
                float gain;
                int num_joints;
                std::string root_frame;
                
        public:
                RightArmModel(ros::NodeHandle &n, std::string frame);
                void setInternalGoal(const VectorT &g);
                void setExternalGoal(const VectorT &g);
                void dynamics(const VectorT &x, const VectorT &u, VectorT &xNext);
                void dynamicsD(const VectorT &x, const VectorT &u, Deriv &d);
                float cost(const VectorT &x, const VectorT &u);

                void fk(const VectorT &x, VectorT &res);
                void ik(const VectorT &pos, VectorT &x);
                
        };
}
#endif
