#!/usr/bin/python

import roslib; roslib.load_manifest('reactive_grasping_pr2')
import rospy
from interactive_markers.interactive_marker_server import *
import copy

from interactive_markers.menu_handler import *

from geometry_msgs.msg import Pose
from motld.msg import TrackedObjects

class SimulatedGoals:
    def __init__(self):
        
        self.obj_pub = rospy.Publisher("objects", TrackedObjects)

        self.last_green = None
        self.last_red = None
        self.server = InteractiveMarkerServer("simulated_goals")
        self.makeMarker( 'red', [1,0,0] )
        self.makeMarker( 'green', [0,1,0] )
        self.server.applyChanges()


    def processFeedback( self, feedback ):
        obj = TrackedObjects()
        rospy.loginfo("feedback")
        obj.name = [1, 2]
        obj.width = [0, 0]
        obj.height = [0, 0]
        obj.pose.header = feedback.header
        obj.pose.header.stamp = rospy.Time.now()
        obj.pose.poses = []

        ps = Pose()
        ps.position.x = feedback.pose.position.x
        ps.position.y = feedback.pose.position.y
        ps.position.z = feedback.pose.position.z
        ps.orientation.w = 1.;

        if feedback.marker_name == "red":
                rospy.loginfo("publishing")
                if self.last_green:
                        obj.pose.poses.append(self.last_green)
                else:
                        obj.pose.poses.append(Pose())
                obj.pose.poses.append(ps)
                self.obj_pub.publish(obj)
                self.last_red = ps
        elif feedback.marker_name == "green":
                rospy.loginfo("publishing")
                obj.pose.poses.append(ps)
                if self.last_red:
                        obj.pose.poses.append(self.last_red)
                else:
                        obj.pose.poses.append(Pose())
                self.obj_pub.publish(obj)
                self.last_green = ps
        else:
                rospy.loginfo("invalid marker name %s" % (feedback.marker_name))

        #print feedback	
        self.server.applyChanges()

    def makeMarker( self, name, color ):
        int_marker = InteractiveMarker()
        int_marker.header.frame_id = "/head_mount_kinect_rgb_optical_frame" #"/openni_rgb_optical_frame"
        int_marker.scale = 1
        int_marker.name = name
        int_marker.description = name

        marker = Marker()

        marker.type = Marker.SPHERE
        marker.scale.x = 0.05
        marker.scale.y = 0.05
        marker.scale.z = 0.05
        marker.color.r = color[0]
        marker.color.g = color[1]
        marker.color.b = color[2]
        marker.color.a = 1.0

        control =  InteractiveMarkerControl()
        control.always_visible = True
        control.markers.append( marker )

        int_marker.controls.append( control )

        control = InteractiveMarkerControl()
        control.orientation.w = 1
        control.orientation.x = 1
        control.orientation.y = 0
        control.orientation.z = 0
        control.name = "move_x"
        control.interaction_mode = InteractiveMarkerControl.MOVE_AXIS
        int_marker.controls.append(control)

        control = InteractiveMarkerControl()
        control.orientation.w = 1
        control.orientation.x = 0
        control.orientation.y = 1
        control.orientation.z = 0
        control.name = "move_z"
        control.interaction_mode = InteractiveMarkerControl.MOVE_AXIS
        int_marker.controls.append(control)

        control = InteractiveMarkerControl()
        control.orientation.w = 1
        control.orientation.x = 0
        control.orientation.y = 0
        control.orientation.z = 1
        control.name = "move_y"
        control.interaction_mode = InteractiveMarkerControl.MOVE_AXIS
        int_marker.controls.append(control)

        self.server.insert(int_marker, self.processFeedback)



if __name__=="__main__":
    rospy.init_node("simulated_goals")
    ball_server = SimulatedGoals()

    rospy.spin()

